unit hiGP_Canvas;

interface

uses Windows,Kol,Share,Debug,GDIPOBJ,GDIPAPI;

type
     TDCMode = (dcBitmap,dcHandle,dcContext);

     TICurrentCanvas = record
      Get_Canvas: function: TGPGraphics of object; 
      Get_Pen: function (name:string): TGPPen of object;  
      Get_Brush: function (name:string):TGPBrush of object; 
      Get_Matrix: function:TGPMatrix of object; 
      Set_Pen: procedure (Pen:TGPPen; name:string) of object;   
      Set_Brush: procedure (Brush:TGPBrush; name:string) of object; 
      Set_Matrix: procedure of object; 
     end;
     PICurrentCanvas = ^TICurrentCanvas;

  THIGP_Canvas = class(TDebug)
   private
     CurrentCanvas:TICurrentCanvas;
     ListPen:PStrList;
     ListBrush:PStrList;
     ArrPen:array of TGPPen;
     ArrBrush:array of TGPBrush;
     Graphics:TGPGraphics; 
     GPMatrix:TGPMatrix;
     UseMatrix:boolean;

     function Get_Canvas:TGPGraphics;
     function Get_Pen (name:string):TGPPen;
     function Get_Brush(name:string):TGPBrush;
     function Get_Matrix:TGPMatrix;
     procedure Set_Pen(Pen:TGPPen; name:string);
     procedure Set_Brush(Brush:TGPBrush; name:string);
     procedure Set_Matrix;

   public
     fDrawSource: TDCMode;

     _prop_Name:string;
     _prop_SmoothingMode:integer;
     _prop_PixelOffsetMode:integer;
     _prop_CompositingMode:integer;
     _prop_CompositingQuality:integer;
     _prop_InterpolationMode:integer;     
     _prop_TextRenderingHint:integer;  
     _data_Bitmap:THI_Event;
     _event_onSet:THI_Event;

     function getInterfaceCurrentCanvas:PICurrentCanvas;
     procedure SetDrawSource(Value:byte);
     procedure _work_doSet(var _Data:TData; Index:word);
     property  _prop_DrawSource: byte write SetDrawSource;
     procedure _work_doDrawSource(var _Data:TData; Index:word);
     procedure _work_doSmoothingMode(var _Data:TData; Index:word);
     procedure _work_doPixelOffsetMode(var _Data:TData; Index:word);
     procedure _work_doCompositingMode(var _Data:TData; Index:word);
     procedure _work_doCompositingQuality(var _Data:TData; Index:word);
     procedure _work_doInterpolationMode(var _Data:TData; Index:word);
     procedure _work_doTextRenderingHint(var _Data:TData; Index:word);
     procedure _var_DpiX(var _Data:TData; Index:word);
     procedure _var_DpiY(var _Data:TData; Index:word);          
  end;

implementation

function THIGP_Canvas.getInterfaceCurrentCanvas:PICurrentCanvas;
begin
  CurrentCanvas.Get_Canvas := Get_Canvas;  
  CurrentCanvas.Get_Pen := Get_Pen; 
  CurrentCanvas.Get_Brush := Get_Brush;
  CurrentCanvas.Get_Matrix := Get_Matrix;
  CurrentCanvas.Set_Pen := Set_Pen;
  CurrentCanvas.Set_Brush := Set_Brush;
  CurrentCanvas.Set_Matrix := Set_Matrix; 
  Result := @CurrentCanvas;
end;


function THIGP_Canvas.Get_Canvas;
begin
  Result := Graphics;
end;

function THIGP_Canvas.Get_Pen(name:string):TGPPen;
var ind:integer;
begin
  ind := ListPen.IndexOf(name);
  if ind = (-1) then ind := 0;  
  Result := ArrPen[ind];
end;

function THIGP_Canvas.Get_Brush(name:string):TGPBrush;
var ind : integer;
begin
  ind := ListBrush.IndexOf(name);
  if ind = (-1) then ind := 0; 
  Result := ArrBrush[ind];
end;

function THIGP_Canvas.Get_Matrix;
begin
  if not UseMatrix then GPMatrix := TGPMatrix.Create();
  Result := GPMatrix;
end;

procedure THIGP_Canvas.Set_Pen(Pen:TGPPen; name:string);
var ind:integer;
begin
 ind := ListPen.IndexOf(name);
 if ind = (-1) then begin
  ind := Length(ArrPen);
  SetLength(ArrPen,ind + 1);
  ListPen.Add(name);
 end 
 else ArrPen[ind].Free;
 ArrPen[ind] := Pen;
end;

procedure THIGP_Canvas.Set_Brush(Brush:TGPBrush; name:string);
var ind:integer;
begin
 ind := ListBrush.IndexOf(name);
 if ind = (-1) then begin
  ind := Length(ArrBrush);
  SetLength(ArrBrush,ind + 1);
  ListBrush.Add(name);
 end 
 else ArrBrush[ind].Free;
 ArrBrush[ind] := Brush;
end;

procedure THIGP_Canvas.Set_Matrix;
begin
  if not UseMatrix then begin
   UseMatrix := true;
   GPMatrix := TGPMatrix.Create;
  end;
end;

procedure THIGP_Canvas.SetDrawSource;
begin
   fDrawSource := TDCMode(Value);
end;

procedure THIGP_Canvas._work_doDrawSource;
begin
   SetDrawSource(ToInteger(_Data));
end;

procedure THIGP_Canvas._work_doSmoothingMode;
begin
 _prop_SmoothingMode := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doPixelOffsetMode;
begin
 _prop_PixelOffsetMode := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doCompositingMode;
begin
 _prop_CompositingMode := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doCompositingQuality;
begin
 _prop_CompositingQuality := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doInterpolationMode;
begin
 _prop_InterpolationMode := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doTextRenderingHint;
begin
 _prop_TextRenderingHint := ToInteger(_Data);  
end;

procedure THIGP_Canvas._work_doSet;
var i:integer;
    bmp:PBitmap;
begin
  case fDrawSource of
   dcHandle:  graphics := TGPGraphics.Create(ReadInteger(_Data,_data_Bitmap,0),false); 
   dcBitmap:  begin
               bmp := ReadBitmap(_Data,_data_Bitmap,nil);
               if (bmp <> nil) and (not bmp.Empty) then
                graphics := TGPGraphics.Create(bmp.Canvas.Handle) 
               else exit;
              end;
   dcContext: graphics := TGPGraphics.Create(ReadInteger(_Data,_data_Bitmap,0),0);
  end;
    UseMatrix := False;
    ListPen := NewStrList;
    ListBrush := NewStrList;
    
    ListPen.Add('0');
    ListBrush.Add('0');

    SetLength(ArrPen,1);
    SetLength(ArrBrush,1);

    ArrPen[0] := TGPPen.Create(aclBlack,-1);
    ArrBrush[0] := TGPSolidBrush.Create(aclWhite); 
    
    graphics.SetSmoothingMode(_prop_SmoothingMode);
    graphics.SetPixelOffsetMode(_prop_PixelOffsetMode);
    graphics.SetCompositingMode(TCompositingMode(_prop_CompositingMode));
    graphics.SetCompositingQuality(_prop_CompositingQuality);
    graphics.SetInterpolationMode(_prop_InterpolationMode);        
    graphics.SetTextRenderingHint(TTextRenderingHint(_prop_TextRenderingHint)); 
    _hi_onEvent(_event_onSet);
    
    for i := 0 to (Length(ArrPen) - 1) do ArrPen[i].Free;
    for i := 0 to (Length(ArrBrush) - 1) do ArrBrush[i].Free;
    SetLength(ArrPen,0);
    SetLength(ArrBrush,0);
    ListPen.Free;
    ListBrush.Free;
    if UseMatrix then GPMatrix.Free;
    graphics.Free;
end;

procedure THIGP_Canvas._var_DpiX;
begin
   if Assigned(graphics) then  dtReal(_Data, graphics.GetDpiX);
end;

procedure THIGP_Canvas._var_DpiY;
begin
   if Assigned(graphics) then  dtReal(_Data, graphics.GetDpiY);
end;


end.